package com.gic.spark.tag;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.gic.spark.datasource.DataSourceManager;
import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlDatasource;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.datasource.sharding.ShardingRddManager;
import com.gic.spark.entity.SceneCrowdDTO;
import com.gic.spark.entity.TagConditionDTO;
import com.gic.spark.entity.TagConditionGroupDTO;
import com.gic.spark.entity.request.FilterRequest;
import com.gic.spark.entity.table.TabMemberTagAll;
import com.gic.spark.entity.table.TabSceneCrowd;
import com.gic.spark.filter.BaseTagFilter;
import com.gic.spark.util.DingtalkMessageUtil;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.sql.Column;
import org.apache.spark.sql.Dataset;
import scala.Tuple2;

import java.util.*;

import static com.gic.spark.datasource.mysql.MysqlDatasource.JDBC_OPTIONS;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/4/15
 */
public class TagProcess {

    private List<SceneCrowdDTO>sceneCrowdDTOList=new ArrayList();
    private  MysqlRddManager mysqlRddManager;
    private  ShardingRddManager memberShardingRddManager;
    private boolean isProduction;
    private static TagProcess instance;
    public static TagProcess getInstance(){
        if(null==instance){
            instance=new TagProcess();
        }
        return instance;
    }
    private TagProcess(){}

    public boolean isProduction() {
        return isProduction;
    }

    public void init(boolean isProd){
        this.isProduction=isProd;
        MysqlDatasource member4Datasource = new MysqlDatasource();
        if(isProduction){
            member4Datasource.setJdbcUrl("jdbc:mysql://10.0.0.6:3306/gic-member4.0" + JDBC_OPTIONS);
            member4Datasource.setUser("cdb_outerroot");
            member4Datasource.setPassword("@09ui%sbc09");
        }else{
            member4Datasource.setJdbcUrl("jdbc:mysql://10.0.0.6:3306/gic-member4.0" + JDBC_OPTIONS);
            member4Datasource.setUser("cdb_outerroot");
            member4Datasource.setPassword("@09ui%sbc09");
        }
        mysqlRddManager=member4Datasource.buildRddManager();

        Dataset<TabSceneCrowd>sceneCrowdDataset=mysqlRddManager.getPojo("", TabSceneCrowd.class,null)
                                                                .filter(new Column("valid_flag").equalTo(1));
        Dataset<TabMemberTagAll>memberTagAllDataset= mysqlRddManager.getPojo("tab_member_tag_all", TabMemberTagAll.class,null);
        List<TabSceneCrowd> sceneCrowdList=sceneCrowdDataset.collectAsList();
        for(TabSceneCrowd sceneCrowd:sceneCrowdList){
            LinkedList<TagConditionGroupDTO>conditionGroupDTOList=JSONObject.parseObject(sceneCrowd.getTag_Condition_Group_Info(),  new TypeReference<LinkedList<TagConditionGroupDTO>>(){});
            sceneCrowdDTOList.add(new SceneCrowdDTO(sceneCrowd,conditionGroupDTOList));
        }

    }

    public void process(boolean extractData){

        Map<Long, List<TagProcessEntity>> tagGroupByEnterpriseMap = new HashMap<>();
        Map<Long, BaseTagFilter> tagIdToFilterMap = new HashMap();
        for(SceneCrowdDTO sceneCrowdDTO:sceneCrowdDTOList){
            if(sceneCrowdDTO.getReal_Time()==1){
                LinkedList<TagConditionGroupDTO> conditionGroupDTOS=sceneCrowdDTO.getConditionGroupDTOList();
                for(int i=0;i<conditionGroupDTOS.size();i++){

                    TagProcessEntity entity=new TagProcessEntity();
                    entity.enterpriseId=sceneCrowdDTO.getEnterprise_Id();
                    entity.tagGroupId=sceneCrowdDTO.getId();
                    entity.level=i+1;

                    entity.tagList=conditionGroupDTOS.get(i).getConditionInfos();
                    for(TagConditionDTO conditionDTO:entity.tagList){//将tag同filter进行映射
                        BaseTagFilter tagFilter=getTagFilter(conditionDTO.getTagEsFieldName());
                        if(null!=tagFilter){
                            tagIdToFilterMap.put(conditionDTO.getTagId(),tagFilter);
                        }
                    }
                    if(!tagGroupByEnterpriseMap.containsKey(sceneCrowdDTO.getEnterprise_Id())){
                        tagGroupByEnterpriseMap.put(sceneCrowdDTO.getEnterprise_Id(),new ArrayList());
                    }
                    tagGroupByEnterpriseMap.get(sceneCrowdDTO.getEnterprise_Id()).add(entity);
                }
            }
        }

        //准备标签需要的数据
        for (Map.Entry<Long, List<TagProcessEntity>> enterpriseTagEntry : tagGroupByEnterpriseMap.entrySet()) {
            for (TagProcessEntity processEntity : enterpriseTagEntry.getValue()) {
                for (TagConditionDTO conditionDTO : processEntity.tagList) {
                    if (tagIdToFilterMap.containsKey(conditionDTO.getTagId())) {
                        for (DataSourceEntity sourceEntity : tagIdToFilterMap.get(conditionDTO.getTagId()).necessarySourceList()) {
                            DataSourceManager.getInstance().addSourceEntity(sourceEntity, enterpriseTagEntry.getKey().intValue());
                        }
                    }
                }
            }
        }

        if (extractData) {
            DataSourceManager.getInstance().extractDataToDatabase();
            DingtalkMessageUtil.sendAlertMessage("extractData to hive finish !");
        }

        //处理标签数据
        for (Map.Entry<Long, List<TagProcessEntity>> enterpriseTagEntry : tagGroupByEnterpriseMap.entrySet()) {
            Long enterpriseId=enterpriseTagEntry.getKey();
            List<JavaPairRDD<String, String>> filterRddList = new ArrayList<>();
            for(TagProcessEntity entity:enterpriseTagEntry.getValue()){
                for(TagConditionDTO conditionDTO:entity.tagList){
                    if(tagIdToFilterMap.containsKey(conditionDTO.getTagId())){
                        BaseTagFilter tagFilter=tagIdToFilterMap.get(conditionDTO.getTagId());
                        FilterRequest filterRequest=TagValueParser.parseFilterValue(conditionDTO);
                        final String groupId = entity.tagGroupId + "_" + conditionDTO.getTagId() + "_" + entity.level;

                        JavaPairRDD<String, String> rdd = tagFilter.filterValidMember(enterpriseId,filterRequest).mapToPair(data-> Tuple2.apply(data,groupId));
                        filterRddList.add(rdd);
                    }
                }
            }

            if(filterRddList.size()>0){
                JavaPairRDD<String,String>memberGroupRdd=null;
                for(JavaPairRDD<String,String>rdd:filterRddList){
                    if(null==memberGroupRdd){
                        rdd=memberGroupRdd;
                    }else{
                        memberGroupRdd.union(rdd);
                    }
                }
            }
        }
    }

    private static BaseTagFilter getTagFilter(String tagEsFieldName){
        BaseTagFilter tagFilter=null;
        switch (tagEsFieldName) {

            //卡卷类
            case TagConstant.TAG_CODE_CURRENT_COUPON_NUM:
                break;
            case TagConstant.TAG_CODE_UNCLAIMED_COUPON:
                break;
            case TagConstant.TAG_CODE_GET_NO_CANCEL_COUPON:
                break;
            case TagConstant.TAG_CODE_NO_CANCEL_EXPIRES_COUPON:
                break;
            case TagConstant.TAG_CODE_CANCEL_COUPON:
                break;
            case TagConstant.TAG_CODE_DONATIONINCOUPON:
                break;
            case TagConstant.TAG_CODE_YETDONATIONCOUPON:
                break;
            default:
                DingtalkMessageUtil.sendAlertMessage("暂未支持的非实时标签: " + tagEsFieldName);
                break;
        }
        return tagFilter;
    }
}
